<?php
/**
 * Created by HOMEMADE.IO SAS.
 * Afex Shipping - Data Reconciliation
 * /**************************************************************************
 * Copyright (C) HOMEMADE.IO SAS, Inc - All Rights Reserved
 *
 * @author          HOMEMADE.IO <contact@home-made.io>
 * @copyright       2013-2024 HOMEMADE.IO SAS
 * @date            2024-05-17
 */

include_once _PS_MODULE_DIR_ . 'afex_shipping/classes/AfexDataReconciliation.php';

class AdminFileResultsController extends ModuleAdminController
{
    public function __construct()
    {
        $this->name = 'AdminFileResults';
        $this->className = 'AfexDataReconciliation';
        $this->table = 'afex_shipping_data_reconciliation';
        $this->identifier = 'id_afex_data_reconciliation';
        $this->bootstrap = true;

        $this->addRowAction('view', 'viewOrder');
        $this->addRowAction('delete');
        $this->bulk_actions = array('delete' => array('text' => 'Delete', 'confirm' => 'Delete selected items?'));

        $this->fields_list = array(
            'id_afex_data_reconciliation' => array(
                'title' => 'ID',
                'type' => 'int',
                'orderby' => true,
                'search' => false,
            ),
            'order_reference' => array(
                'title' => 'Order Reference',
                'type' => 'text',
                'orderby' => true,
                'search' => true,
            ),
            'carrier_awb' => array(
                'title' => 'AWB Number',
                'type' => 'int',
                'orderby' => true,
                'search' => true,
            ),
            'initial_status' => array(
                'title' => 'Initial Status',
                'type' => 'text',
                'orderby' => false,
                'search' => false,
            ),
            'post_process_status' => array(
                'title' => 'Processed Status',
                'type' => 'text',
                'orderby' => false,
                'search' => false,
            ),
            'status' => array(
                'title' => 'OK',
                'type' => 'text',
                'orderby' => true,
                'search' => true,
                'callback' => 'printStatus',
            ),
            'note' => array(
                'title' => 'Note',
                'type' => 'text',
            ),
            'date_time' => array(
                'title' => 'Date',
                'type' => 'datetime',
                'orderby' => true,
                'search' => true,
            ),
        );

        $this->orderBy = 'date_time';
        $this->orderWay = 'DESC';

        parent::__construct();
    }

    public function initProcess()
    {
        parent::initProcess();

        if (Tools::isSubmit('delete_all')) {
            $this->action = 'delete_all';
        }
    }

    public function processDeleteAll()
    {
        Db::getInstance()->execute('TRUNCATE TABLE ' . _DB_PREFIX_ . $this->table);
        Tools::redirectAdmin(self::$currentIndex . '&token=' . $this->token);
    }

    public function initToolbar()
    {
        parent::initToolbar();
        $this->toolbar_btn['delete'] = array(
            'short' => 'Erase',
            'desc' => 'Delete All Rows',
            'js' => 'if (confirm(\'' . 'Delete All Rows?' . '\')) document.location = \'' .
                Tools::safeOutput($this->context->link->getAdminLink('AdminFileResultsController')) . '&amp;delete_all\';'
        );
        unset($this->toolbar_btn['new']);
    }

    public function renderView()
    {
        $id = (int)Tools::getValue($this->identifier);
        $reconciliation = new AfexDataReconciliation($id);

        if (Validate::isLoadedObject($reconciliation)) {
            $order_reference = $reconciliation->order_reference;

            // Fetch the order ID based on the order reference
            $order_id = (int)Db::getInstance()->getValue('SELECT id_order FROM ' . _DB_PREFIX_ . 'orders WHERE reference = \'' . pSQL($order_reference) . '\'');

            if ($order_id) {
                $url = $this->context->link->getAdminLink('AdminOrders', true, ['id_order' => $order_id, 'vieworder' => '']);
                print_r($url);
                Tools::redirectAdmin($url);
            } else {
                $this->errors[] = $this->l('The order could not be found.');
            }
        } else {
            $this->errors[] = $this->l('The reconciliation record could not be found.');
        }

        return parent::renderView();
    }

     public function printStatus($value, $rule)
    {
        $stateHtml = $value;

        if ($value == 'failed') {
            $css_class = 'badge-danger';
        } elseif ($value == 'successful') {
            $css_class = 'badge-success';
        } else {
            $css_class = 'badge-warning';
        }

        return '<span class="badge ' . $css_class . '" style="padding-right:10px; padding-left:10px">' . $value . '</span>';
    }
}
