<?php

class AdminUploadController extends ModuleAdminController
{
    public function __construct()
    {
        $this->bootstrap = true;
        parent::__construct();
    }

    public function initContent()
    {
        parent::initContent();

        $this->context->smarty->assign([
            'upload_url' => self::$currentIndex . '&token=' . $this->token . '&submitUpload',
            'module_name' => $this->module->displayName, // Assign module name to the template
            'module' => $this->module, // Assign the module instance to the template
        ]);

        $this->setTemplate('upload.tpl');
    }

    public function postProcess()
    {
        if (Tools::isSubmit('submitUpload')) {
            if (isset($_FILES['upload_file']) && !empty($_FILES['upload_file']['tmp_name'])) {
                $fileTmpPath = $_FILES['upload_file']['tmp_name'];
                $fileName = $_FILES['upload_file']['name'];
                $fileSize = $_FILES['upload_file']['size'];
                $fileType = $_FILES['upload_file']['type'];
                $fileNameCmps = explode(".", $fileName);
                $fileExtension = strtolower(end($fileNameCmps));

                $allowedfileExtensions = array('csv', 'xls', 'xlsx');
                if (in_array($fileExtension, $allowedfileExtensions)) {
                    $timestamp = time();
                    $newFileName = $timestamp . '_' . $fileName;
                    $uploadFileDir = _PS_MODULE_DIR_ . $this->module->name . '/uploads/';
                    $dest_path = $uploadFileDir . $newFileName;

                    if (!file_exists($uploadFileDir)) {
                        mkdir($uploadFileDir, 0777, true);
                    }

                    if (move_uploaded_file($fileTmpPath, $dest_path)) {
                        $results = $this->processUploadedFile($dest_path, $fileExtension);
                        Db::getInstance()->insert('afex_shipping_files', [
                            'file_name' => pSQL($newFileName),
                            'type' => pSQL($_POST['file_type'] ?? 'unknown'),
                        ]);
                        $fileId = Db::getInstance()->Insert_ID();
                       
                        $this->saveReconciliationData($fileId, $results);
                        $this->confirmations[] = $this->module->l('File is successfully uploaded.');
                    } else {
                        $this->errors[] = $this->module->l('There was some error moving the file to upload directory. Please make sure the upload directory is writable by web server.');
                    }
                } else {
                    $this->errors[] = $this->module->l('Upload failed. Allowed file types: ' . implode(',', $allowedfileExtensions));
                }
            }
        }
    }

    protected function processUploadedFile($filePath, $fileExtension)
    {
        if ($fileExtension == 'csv') {
            return $this->processCSVFile($filePath);
        } else {
            return $this->processExcelFile($filePath);
        }
    }

    protected function processCSVFile($filePath)
    {
        $results = [
            'successful' => [],
            'failed' => [],
        ];

        if (($handle = fopen($filePath, 'r')) !== FALSE) {
            $headers = fgetcsv($handle, 1000, ",");
            while (($data = fgetcsv($handle, 1000, ",")) !== FALSE) {
            // Process each row in the CSV
            try {
                // Assume the second column is the order reference
                $order_reference = $data[1];
                $orders = Order::getByReference($order_reference);
                
                if ($orders->count()) {
                $order = $orders->getFirst();
                $current_status = (new OrderState($order->current_state))->name[$this->context->language->id];
                try{
                    $status = self::updateOrderStatusAndPayment($order, 5, $data[6], $data[5]);
                    if($status !== null){
                    $results['successful'][] = [
                        'order_id' => $order->id,
                        'order_reference' => $order_reference,
                        'carrier_awb' => $data[0], // Assuming this is the AWB
                        'initial_status' => $current_status,
                        'post_process_status' => 'Processed Status', // Set after processing
                    ];
                    }
                } catch (Exception $e) {
                    $results['failed'][] = [
                    'order_reference' => $order_reference,
                    'error' => $e->getMessage(),
                    'carrier_awb' => $data[0], // Assuming this is the AWB
                    ];
                }
                
                } else {
                throw new Exception('Order not found.');
                }
            } catch (Exception $e) {
                $results['failed'][] = [
                'order_reference' => $order_reference,
                'error' => $e->getMessage(),
                'carrier_awb' => $data[0], // Assuming this is the AWB
                ];
            }
            }
            fclose($handle);
        }
        return $results;
    }

    protected function processExcelFile($filePath)
    {
        $results = [
            'successful' => [],
            'failed' => [],
        ];
        
        require_once(_PS_MODULE_DIR_ . $this->module->name . '/vendor/autoload.php');
        $spreadsheet = \PhpOffice\PhpSpreadsheet\IOFactory::load($filePath);
        $worksheet = $spreadsheet->getActiveSheet();
        foreach ($worksheet->getRowIterator() as $row) {
            $cellIterator = $row->getCellIterator();
            $cellIterator->setIterateOnlyExistingCells(false);
            $order_reference = null;

            foreach ($cellIterator as $cell) {
                if (!$order_reference) {
                    $order_reference = $cell->getValue();
                }
            }

            if ($order_reference) {
                try {
                    $orders = Order::getByReference($order_reference);

                    if ($orders->count()) {
                        $order = $orders->getFirst();
                        $current_status = (new OrderState($order->current_state))->name[$this->context->language->id];
                        $results['successful'][] = [
                            'order_id' => $order->id,
                            'order_reference' => $order_reference,
                            'carrier_awb' => $order->shipping_number, // Assuming this is the AWB
                            'initial_status' => $current_status,
                            'post_process_status' => 'Processed Status', // Set after processing
                        ];
                    } else {
                        throw new Exception('Order not found.');
                    }
                } catch (Exception $e) {
                    $results['failed'][] = [
                        'order_reference' => $order_reference,
                        'error' => $e->getMessage(),
                    ];
                }
            }
        }

        return $results;
    }

    protected function saveReconciliationData($fileId, $results)
    {
        if (empty($results['successful']) && empty($results['failed'])) {
            return;
        }
        
        if (!empty($results['successful'])) {
            foreach ($results['successful'] as $successful) {
                Db::getInstance()->insert('afex_shipping_data_reconciliation', [
                    'id_afex_shipping_file' => (int) $fileId,
                    'order_id' => (int) $successful['order_id'],
                    'order_reference' => pSQL($successful['order_reference']),
                    'carrier_awb' => pSQL($successful['carrier_awb']),
                    'initial_status' => pSQL($successful['initial_status']),
                    'post_process_status' => pSQL($successful['post_process_status']),
                    'status' => 'successful',
                    'note' => '',
                ]);
            }
        }
        
        if (!empty($results['failed'])) {
            foreach ($results['failed'] as $failed) {
                Db::getInstance()->insert('afex_shipping_data_reconciliation', [
                    'id_afex_shipping_file' => (int) $fileId,
                    'order_id' => 0,
                    'order_reference' => pSQL($failed['order_reference']),
                    'carrier_awb' => pSQL($failed['carrier_awb']),
                    'initial_status' => '',
                    'post_process_status' => '',
                    'status' => 'failed',
                    'note' => pSQL($failed['error']),
                ]);
            }
        }
       
    }

    public function getOrderObjectByNumber($order_reference)
    {
        $orders = Order::getByReference($order_reference);
        if ($orders->count()) {
            $order = $orders->getFirst();
            $current_status_id = $order->current_state;
            $order_state = new OrderState($current_status_id);
            $current_status_name = $order_state->name[$this->context->language->id];
            // Output the current order status name
            return 'The current status of order ' . $order_reference . ' is: ' . $current_status_name;
        } else {
            return 'Order not found.';
        }
    }

    public function updateOrderStatusAndPayment($order, $newStatusId, $chequeNumber, $paymentOrder)
    {
        try {
            if (!Validate::isLoadedObject($order)) {
                throw new Exception('Order not found.');
            }

            // Check if the order already has payments and its status is equal to newStatusId
            $existingPayments = OrderPayment::getByOrderReference($order->reference);
            foreach ($existingPayments as $existingPayment) {
                $existingStatus = OrderHistory::getLastOrderState($order->id);
                if ((int)$existingStatus->id === (int)$newStatusId) {
                    return null; // Return nothing if the conditions are met
                }
            }

            // Change order status
            $history = new OrderHistory();
            $history->id_order = (int)$order->id;
            $history->changeIdOrderState((int)$newStatusId, $order, true);
            $history->addWithemail(true, [
                'order_name' => $order->getUniqReference()
            ]);

            // Remove existing payments to ensure no duplicates
            foreach ($existingPayments as $existingPayment) {
                $existingPayment->delete();
            }

            // Assign payment method as cheque
            $payment = new OrderPayment();
            $payment->order_reference = $order->reference;
            $payment->id_currency = (int)Configuration::get('PS_CURRENCY_DEFAULT');
            $payment->amount = $order->total_paid_tax_incl; // Ensure this matches the order's total paid amount
            $payment->payment_method = 'Cheque';
            $payment->transaction_id = $chequeNumber;
            $payment->card_number = $paymentOrder; // Assuming this is where you want to store Payment Order
            $payment->add();

            // Optionally link the payment to the order, if not already done by $payment->add()
            // Comment out the following line if it is redundant
            // $order->addOrderPayment($order->total_paid, 'Cheque', $chequeNumber, null, null, $paymentOrder);

            return [
                'status' => 'success',
                'message' => 'Order status and payment method updated successfully.'
            ];
        } catch (Exception $e) {
            return [
                'status' => 'error',
                'message' => $e->getMessage()
            ];
        }
    }
}